/**
 * 将对象的snake属性转换为camle格式
 * @param source 源对象
 * @param target 目标对象，可选
 * @returns  转换后的对象
 *
 * @example
 * const obj = {
 *  'user_name': 'dinos',
 *  'user_age': 18,
 * }
 *
 * const res = toCamleObject(obj)
 * // res = {
 * //  userName: 'dinos',
 * //  userAge: 18,
 * // }
 */
export declare const toCamleObject: <T>(source: Object, target?: T) => T;
/**
 * 将对象的camle属性转换为snake格式
 * @param source 源对象
 * @param target 目标对象，可选
 * @returns 转换后的对象
 *
 * @example
 * const obj = {
 *   userName: 'dinos',
 *   userAge: 18,
 * }
 *
 * const res = toSnakeObject(obj)
 * // res = {
 * //  'user_name': 'dinos',
 * //  'user_age': 18,
 * // }
 */
export declare const toSnakeObject: <T>(source: any, target?: T) => T;
/**
 * 将'T | T[]' 这种类型统一处理成 T[]类型
 * @param v 待处理的值或数组
 * @returns T[]
 *
 * @example
 * const a = asArray('a')
 * // a = ['a']
 *
 * const b = asArray(['a', 'b'])
 * // b = ['a', 'b']
 */
export declare const asArray: <T>(v: T | T[]) => T[];
/**
 * 同步获取值或函数调用结果
 * @param v 值，函数或Promise对象
 * @param restArgs 函数调用时的参数
 * @returns
 *
 * @example
 * const a = awaitGet('a')
 * // a = 'a'
 *
 * const b = awaitGet(() => 'b')
 * // b = 'b'
 *
 * const c = awaitGet(Promise.resolve('c'))
 * // c = 'c'
 *
 * const d = awaitGet(() => Promise.resolve('d'))
 * // d = 'd'
 *
 * const e = awaitGet((rest) => Promise.resolve(rest), 'e')
 * // e = 'e'
 */
export declare const awaitGet: <T>(v: T | Promise<T> | ((...args: any[]) => Promise<T>) | ((...args: any[]) => T), ...restArgs: any[]) => T;
/**
 * 是否是Promise对象
 * @param val 待判断的值
 * @returns 是否是Promise对象
 */
export declare const isPromise: <T = any>(val: unknown) => val is Promise<T>;
/**
 * 生成不断增长的id
 * @param prefix 前缀
 * @returns prefix + id
 *
 * @example
 * const a = useID()
 * // a = '1'
 *
 * const b = useID('dino-')
 * // b = 'dino-2'
 */
export declare const useID: (prefix?: string) => string;
